package org.rcisoft.sys.attendance.schedule;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.rcisoft.sys.attendance.dao.AttendanceSyncRepository;
import org.rcisoft.sys.attendance.dao.ScheduledResultRepository;
import org.rcisoft.sys.attendance.entity.Attendance;
import org.rcisoft.sys.attendance.entity.AttendanceSync;
import org.rcisoft.sys.attendance.entity.ScheduledResult;
import org.rcisoft.sys.attendance.service.AttendanceService;
import org.rcisoft.sys.attendance.util.TimeUtils;
import org.rcisoft.sys.rbac.role.entity.SysRoleRbac;
import org.rcisoft.sys.sysuser.dao.SysUserRepositorys;
import org.rcisoft.sys.sysuser.entity.SysUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Component
@ConditionalOnProperty(prefix = "syncSchedule", name = "config", havingValue = "true")
public class AttendanceSyncSchedule {
    @Autowired
    private AttendanceService attendanceService;
    @Autowired
    private SysUserRepositorys sysUserRepositorys;

    @Autowired
    private AttendanceSyncRepository attendanceSyncRepository;

    @Autowired
    private ScheduledResultRepository scheduledResultRepository;

    //    每天凌晨1点执行一次
    @Scheduled(cron = "0 0 1/3 * * ?")
//    @Scheduled(cron = "0 0/2 * * * ?")
    public void execute() {
        Date update = TimeUtils.getUpdate();
        QueryWrapper<ScheduledResult> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("type","考勤同步");
        queryWrapper.eq("kq_date", TimeUtils.getDateNotTime(update));
        queryWrapper.eq("flag", "1");
        List<ScheduledResult> list = scheduledResultRepository.selectList(queryWrapper);
        if (list.isEmpty()) {
            queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("kq_date", TimeUtils.getDateNotTime(update));
            List<ScheduledResult> list1 = scheduledResultRepository.selectList(queryWrapper);
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            try {
                Attendance attendance = new Attendance();
                // 查询昨日所有考勤
                List<Attendance> all = attendanceService.findAll(attendance);
                AttendanceSync attendanceSync = new AttendanceSync();
                for (int i = 0; i < all.size(); i++) {
                    //通过id查询用户表
                    SysUser sysUser = sysUserRepositorys.querySysUsersById(all.get(i).getUserId());
                    // TODO: 2022/12/30 查询用户的非空校验
                    if (sysUser != null) {
                        attendanceSync.setUserId(sysUser.getBusinessId());
                        attendanceSync.setName(all.get(i).getPeopleName());
                        attendanceSync.setPhone(sysUser.getPhone());
                        attendanceSync.setDeptId(sysUser.getDeptId());
                        attendanceSync.setDeptLabel(all.get(i).getDeptName());
                        attendanceSync.setPosition(sysUser.getPosition());
                        attendanceSync.setPositionLabel(all.get(i).getPositionName());
                        attendanceSync.setWorkHours(all.get(i).getWorkShift());
                        attendanceSync.setOffHours(all.get(i).getClosingTime());
                        attendanceSync.setWorkDeviceName(all.get(i).getWorkDeviceName());
                        attendanceSync.setOffDeviceName(all.get(i).getOfficeDeviceName());
                        attendanceSync.setDateOfAttendance(all.get(i).getKqDate());
                        attendanceSync.setTimeOnJob(all.get(i).getDuration());
                        attendanceSync.setNltBz(sysUser.getNltBz());
                        attendanceSync.setNltBzLabel(all.get(i).getNltBz());
                        attendanceSync.setFlag(all.get(i).getKqFlag());
                        attendanceSync.setCreateDate(new Date());
                        attendanceSync.setPassageTime(all.get(i).getCrossTime());
                        if (attendanceSync.getFlag().equals("正常")) {
                            attendanceSync.setFlag("1");
                        } else if (attendanceSync.getFlag().equals("异常")) {
                            attendanceSync.setFlag("0");
                        }
                        // 判断考勤是否已经存在，已经存在不会添加
                        QueryWrapper<AttendanceSync> wrapper = new QueryWrapper();
                        wrapper.eq("date_of_attendance",attendanceSync.getDateOfAttendance());
                        wrapper.eq("user_id",attendanceSync.getUserId());
                        List<AttendanceSync> attendanceSyncList = attendanceSyncRepository.selectList(wrapper);
                        if(attendanceSyncList.size()==0)
                            attendanceSyncRepository.addAttendanceSync(attendanceSync);
                    }
                }
                SysUser user = new SysUser();
                user.setFlag("1");
                user.setAttendanceShow("0");
                //查询所有在职，显示考勤的用户
                List<SysUser> users = sysUserRepositorys.querySysUsers(user);
                List<Integer> userIds = all.stream().map(Attendance::getUserId).collect(Collectors.toList());
                users = users.stream().filter(u -> !userIds.contains(u.getBusinessId())).collect(Collectors.toList());
                for (SysUser sysUser : users) {
                    attendanceSync = new AttendanceSync();
                    attendanceSync.setUserId(sysUser.getBusinessId());
                    attendanceSync.setName(sysUser.getName());
                    attendanceSync.setPhone(sysUser.getPhone());
                    attendanceSync.setDeptId(sysUser.getDeptId());
                    attendanceSync.setDeptLabel(sysUser.getDeptName());
                    attendanceSync.setPosition(sysUser.getPosition());
                    attendanceSync.setPositionLabel(sysUser.getPositionLabel());
                    attendanceSync.setWorkHours(null);
                    attendanceSync.setOffHours(null);
                    attendanceSync.setWorkDeviceName(null);
                    attendanceSync.setOffDeviceName(null);
                    attendanceSync.setDateOfAttendance(sdf.format(update));
                    attendanceSync.setTimeOnJob(null);
                    attendanceSync.setNltBz(sysUser.getNltBz());
                    attendanceSync.setNltBzLabel(sysUser.getNltBzLabel());
                    attendanceSync.setCreateDate(new Date());
                    attendanceSync.setPassageTime(null);
                    attendanceSync.setFlag("0");
                    // 判断考勤是否已经存在，已经存在不会添加
                    QueryWrapper<AttendanceSync> wrapper = new QueryWrapper();
                    wrapper.eq("date_of_attendance",attendanceSync.getDateOfAttendance());
                    wrapper.eq("user_id",attendanceSync.getUserId());
                    List<AttendanceSync> attendanceSyncList = attendanceSyncRepository.selectList(wrapper);
                    if(attendanceSyncList.size()==0)
                        attendanceSyncRepository.addAttendanceSync(attendanceSync);
                }
                ScheduledResult scheduledResult = new ScheduledResult();
                scheduledResult.setType("考勤同步");
                scheduledResult.setKqDate(TimeUtils.getDateNotTime(update));
                scheduledResult.setFlag("1");
                if (list1.isEmpty())
                    scheduledResultRepository.insert(scheduledResult);
                else {
                    scheduledResult.setBusinessId(list1.get(0).getBusinessId());
                    scheduledResultRepository.updateById(scheduledResult);
                }
            } catch (Exception e) {
                ScheduledResult scheduledResult = new ScheduledResult();
                scheduledResult.setType("考勤同步");
                scheduledResult.setKqDate(TimeUtils.getDateNotTime(update));
                scheduledResult.setFlag("0");
                if (list1.isEmpty())
                    scheduledResultRepository.insert(scheduledResult);
            }
        }
    }
}
