package org.rcisoft.sys.attendance.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.aop.CyPageUtilAsp;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.rcisoft.core.service.CyBaseService;

import org.rcisoft.sys.attendance.dao.AttendanceSyncRepository;
import org.rcisoft.sys.attendance.entity.AttendanceSync;
import org.rcisoft.sys.attendance.service.AttendanceSyncService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.rcisoft.core.model.CyPageInfo;

import java.math.BigDecimal;
import java.util.List;
import java.util.stream.Collectors;

import lombok.extern.slf4j.Slf4j;

/**
 * Created by cy on 2022年11月28日 下午4:57:21.
 */
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class AttendanceSyncServiceImpl extends ServiceImpl<AttendanceSyncRepository, AttendanceSync> implements AttendanceSyncService {


    /**
     * 保存 考勤同步表
     *
     * @param attendanceSync
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(AttendanceSync attendanceSync) {
        //增加操作
        int line = baseMapper.insert(attendanceSync);
        log.debug(CyUserUtil.getAuthenUsername() + "新增了ID为" +
                attendanceSync.getBusinessId() + "的考勤同步表信息");
        return new CyPersistModel(line);
    }

    /**
     * 删除 考勤同步表
     *
     * @param attendanceSync
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(AttendanceSync attendanceSync) {
        int line = baseMapper.realDelete(attendanceSync);
        log.debug(CyUserUtil.getAuthenUsername() + "删除了ID为" +
                attendanceSync.getBusinessId() + "的考勤同步表信息");
        return new CyPersistModel(line);
    }

    /**
     * 逻辑删除 考勤同步表
     *
     * @param attendanceSync
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(AttendanceSync attendanceSync) {
        attendanceSync.setDeleted();
        int line = baseMapper.deleteById(attendanceSync);
        log.debug(CyUserUtil.getAuthenUsername() + "逻辑删除了ID为" +
                attendanceSync.getBusinessId() + "的考勤同步表信息");
        return new CyPersistModel(line);
    }

    /**
     * 修改 考勤同步表
     *
     * @param attendanceSync
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(AttendanceSync attendanceSync) {
        int line = baseMapper.updateById(attendanceSync);
        log.debug(CyUserUtil.getAuthenUsername() + "修改了ID为" + attendanceSync.getBusinessId() + "的考勤同步表信息");
        return new CyPersistModel(line);
    }

    /**
     * 根据id查询 考勤同步表
     *
     * @param businessId
     * @return
     */
    @Override
    public AttendanceSync findById(Integer businessId) {
        return baseMapper.selectById(businessId);
    }

    /**
     * 分页查询 考勤同步表
     *
     * @param attendanceSync
     * @return
     */
    @Override
    public IPage<AttendanceSync> findAllByPagination(CyPageInfo<AttendanceSync> paginationUtility,
                                                     AttendanceSync attendanceSync) {

        IPage<AttendanceSync> paged = baseMapper.queryAttendanceSyncsPaged(paginationUtility, attendanceSync);
        // 在岗时长返回处理
        paged.getRecords().stream().map(item -> {
            item.setTimeOnJobLabel(item.getTimeOnJob() != null ? item.getTimeOnJob().toString() + "小时" : null);
            return item;
        }).collect(Collectors.toList());
        return paged;

    }

    /**
     * 查询list 考勤同步表
     *
     * @param attendanceSync
     * @return
     */
    @Override
    public List<AttendanceSync> findAll(AttendanceSync attendanceSync) {
        return baseMapper.queryAttendanceSyncs(attendanceSync);

    }

    /**
     * 导出考勤同步表
     *
     * @return
     */
    @Override
    public List<AttendanceSync> export(AttendanceSync attendanceSync) {
        List<AttendanceSync> attendanceSyncList = baseMapper.queryAttendanceSyncs(attendanceSync);
        // 在岗时长和考勤状态导出处理
        attendanceSyncList.stream().
                map(item -> {
                    item.setTimeOnJobLabel(item.getTimeOnJob() != null ? item.getTimeOnJob().toString() + "小时": null);
                    return item;
                }).
                map(item -> {
                    item.setFlag(item.getFlag().equals("0") ? "异常" : "正常");
                    return item;
                }).collect(Collectors.toList());
        return attendanceSyncList;
    }

}
