package org.rcisoft.sys.attendance.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.model.CyPageInfo;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.sys.attendance.dao.AttendanceSyncRepository;
import org.rcisoft.sys.attendance.dao.ScheduledResultRepository;
import org.rcisoft.sys.attendance.entity.Attendance;
import org.rcisoft.sys.attendance.entity.AttendanceSync;
import org.rcisoft.sys.attendance.entity.ScheduledResult;
import org.rcisoft.sys.attendance.service.AttendanceService;
import org.rcisoft.sys.attendance.service.ScheduledResultService;
import org.rcisoft.sys.attendance.util.TimeUtils;
import org.rcisoft.sys.sysuser.dao.DoorRecordRepository;
import org.rcisoft.sys.sysuser.dao.SysUserRepositorys;
import org.rcisoft.sys.sysuser.entity.DoorRecord;
import org.rcisoft.sys.sysuser.entity.SysUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * Created by cy on 2022年11月28日 下午4:57:21.
 */
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class ScheduledResultServiceImpl extends ServiceImpl<ScheduledResultRepository, ScheduledResult> implements ScheduledResultService {
    @Autowired
    private AttendanceServiceImpl attendanceService;

    @Autowired
    private SysUserRepositorys sysUserRepositorys;

    @Autowired
    private AttendanceSyncRepository attendanceSyncRepository;

    @Autowired
    private ScheduledResultRepository scheduledResultRepository;

    @Autowired
    DoorRecordRepository doorRecordRepository;

    /**
     * 分页查询 考勤同步表
     *
     * @param scheduledResult
     * @return
     */
    @Override
    public IPage<ScheduledResult> findAllByPagination(CyPageInfo<ScheduledResult> paginationUtility,
                                                     ScheduledResult scheduledResult) {
        return baseMapper.queryScheduledResultPaged(paginationUtility, scheduledResult);

    }

    @Override
    public CyPersistModel implement(ScheduledResult scheduledResult) throws ParseException {
//        Date update = TimeUtils.getUpdate();
        List<AttendanceSync> attendanceSyncs = new ArrayList<>();
        QueryWrapper<ScheduledResult> queryWrapper = new QueryWrapper<>();
        if (scheduledResult.getKqDate()!=null){
            queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("kq_date", TimeUtils.getDateNotTime(scheduledResult.getKqDate()));
            List<ScheduledResult> list1 = scheduledResultRepository.selectList(queryWrapper);
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            try {
                Attendance attendance = new Attendance();
                List<Attendance> all = selectAttendanceList(attendance,scheduledResult);
                AttendanceSync attendanceSync = new AttendanceSync();
                for (int i = 0; i < all.size(); i++) {
                    //通过id查询用户表
                    SysUser sysUser = sysUserRepositorys.querySysUsersById(all.get(i).getUserId());
                    // TODO: 2022/12/30 查询用户的非空校验
                    if (sysUser != null) {
                        attendanceSync.setUserId(sysUser.getBusinessId());
                        attendanceSync.setName(all.get(i).getPeopleName());
                        attendanceSync.setPhone(sysUser.getPhone());
                        attendanceSync.setDeptId(sysUser.getDeptId());
                        attendanceSync.setDeptLabel(all.get(i).getDeptName());
                        attendanceSync.setPosition(sysUser.getPosition());
                        attendanceSync.setPositionLabel(all.get(i).getPositionName());
                        attendanceSync.setWorkHours(all.get(i).getWorkShift());
                        attendanceSync.setOffHours(all.get(i).getClosingTime());
                        attendanceSync.setWorkDeviceName(all.get(i).getWorkDeviceName());
                        attendanceSync.setOffDeviceName(all.get(i).getOfficeDeviceName());
                        attendanceSync.setDateOfAttendance(all.get(i).getKqDate());
                        attendanceSync.setTimeOnJob(all.get(i).getDuration());
                        attendanceSync.setNltBz(sysUser.getNltBz());
                        attendanceSync.setNltBzLabel(all.get(i).getNltBz());
                        attendanceSync.setFlag(all.get(i).getKqFlag());
                        attendanceSync.setCreateDate(new Date());
                        attendanceSync.setPassageTime(all.get(i).getCrossTime());
                        if (attendanceSync.getFlag().equals("正常")) {
                            attendanceSync.setFlag("1");
                        } else if (attendanceSync.getFlag().equals("异常")) {
                            attendanceSync.setFlag("0");
                        }
                        // 判断考勤是否已经存在，已经存在不会添加
                        QueryWrapper<AttendanceSync> wrapper = new QueryWrapper();
                        wrapper.eq("date_of_attendance",attendanceSync.getDateOfAttendance());
                        wrapper.eq("user_id",attendanceSync.getUserId());
                        List<AttendanceSync> attendanceSyncList = attendanceSyncRepository.selectList(wrapper);
                        if(attendanceSyncList.size()==0)
                            attendanceSyncs.add(attendanceSync);
//                            attendanceSyncRepository.addAttendanceSync(attendanceSync);
                    }
                }
                SysUser user = new SysUser();
                user.setFlag("1");
                user.setAttendanceShow("0");
                //查询所有在职，显示考勤的用户
                List<SysUser> users = sysUserRepositorys.querySysUsers(user);
                List<Integer> userIds = all.stream().map(Attendance::getUserId).collect(Collectors.toList());
                users = users.stream().filter(u -> !userIds.contains(u.getBusinessId())).collect(Collectors.toList());
                for (SysUser sysUser : users) {
                    attendanceSync = new AttendanceSync();
                    attendanceSync.setUserId(sysUser.getBusinessId());
                    attendanceSync.setName(sysUser.getName());
                    attendanceSync.setPhone(sysUser.getPhone());
                    attendanceSync.setDeptId(sysUser.getDeptId());
                    attendanceSync.setDeptLabel(sysUser.getDeptName());
                    attendanceSync.setPosition(sysUser.getPosition());
                    attendanceSync.setPositionLabel(sysUser.getPositionLabel());
                    attendanceSync.setWorkHours(null);
                    attendanceSync.setOffHours(null);
                    attendanceSync.setWorkDeviceName(null);
                    attendanceSync.setOffDeviceName(null);
                    attendanceSync.setDateOfAttendance(sdf.format(scheduledResult.getKqDate()));
                    attendanceSync.setTimeOnJob(null);
                    attendanceSync.setNltBz(sysUser.getNltBz());
                    attendanceSync.setNltBzLabel(sysUser.getNltBzLabel());
                    attendanceSync.setCreateDate(new Date());
                    attendanceSync.setPassageTime(null);
                    attendanceSync.setFlag("0");
                    // 判断考勤是否已经存在，已经存在不会添加
                    QueryWrapper<AttendanceSync> wrapper = new QueryWrapper();
                    wrapper.eq("date_of_attendance",attendanceSync.getDateOfAttendance());
                    wrapper.eq("user_id",attendanceSync.getUserId());
                    List<AttendanceSync> attendanceSyncList = attendanceSyncRepository.selectList(wrapper);
                    if(attendanceSyncList.size()==0)
                        attendanceSyncs.add(attendanceSync);
//                        attendanceSyncRepository.addAttendanceSync(attendanceSync);
                }
                scheduledResult.setType("考勤同步");
                scheduledResult.setKqDate(TimeUtils.getDateNotTime(scheduledResult.getKqDate()));
                scheduledResult.setFlag("1");
                scheduledResult.setBusinessId(list1.get(0).getBusinessId());
                scheduledResultRepository.updateById(scheduledResult);
            } catch (Exception e) {
                throw new CyServiceException(500,"任务执行失败");
            }
        } else if (StringUtils.isNotBlank(scheduledResult.getBeginDate())&&StringUtils.isNotBlank(scheduledResult.getBeginDate())) {
            List<String> dates = daysBetween(scheduledResult.getBeginDate(),scheduledResult.getEndDate());
            for (String date : dates) {
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
//                try {
                Attendance attendance = new Attendance();
                scheduledResult.setKqDate(sdf.parse(date));
                if (StringUtils.isNotBlank(scheduledResult.getPhone())) {
                    List<SysUser> sysUsers = sysUserRepositorys.querySysUsersByPhone(scheduledResult.getPhone());
                    if (sysUsers.isEmpty())
                        throw new CyServiceException(500,"该手机号不存在");
                    if (sysUsers.size()>1)
                        throw new CyServiceException(500,"该手机号存在多个用户");
                    attendance.setUserId(sysUsers.get(0).getBusinessId());
                }
                List<Attendance> all = selectAttendanceList(attendance,scheduledResult);
                AttendanceSync attendanceSync = new AttendanceSync();
                for (int i = 0; i < all.size(); i++) {
                    //通过id查询用户表
                    SysUser sysUser = sysUserRepositorys.querySysUsersById(all.get(i).getUserId());
                    // TODO: 2022/12/30 查询用户的非空校验
                    if (sysUser != null) {
                        attendanceSync.setUserId(sysUser.getBusinessId());
                        attendanceSync.setName(all.get(i).getPeopleName());
                        attendanceSync.setPhone(sysUser.getPhone());
                        attendanceSync.setDeptId(sysUser.getDeptId());
                        attendanceSync.setDeptLabel(all.get(i).getDeptName());
                        attendanceSync.setPosition(sysUser.getPosition());
                        attendanceSync.setPositionLabel(all.get(i).getPositionName());
                        attendanceSync.setWorkHours(all.get(i).getWorkShift());
                        attendanceSync.setOffHours(all.get(i).getClosingTime());
                        attendanceSync.setWorkDeviceName(all.get(i).getWorkDeviceName());
                        attendanceSync.setOffDeviceName(all.get(i).getOfficeDeviceName());
                        attendanceSync.setDateOfAttendance(all.get(i).getKqDate());
                        attendanceSync.setTimeOnJob(all.get(i).getDuration());
                        attendanceSync.setNltBz(sysUser.getNltBz());
                        attendanceSync.setNltBzLabel(all.get(i).getNltBz());
                        attendanceSync.setFlag(all.get(i).getKqFlag());
                        attendanceSync.setCreateDate(new Date());
                        attendanceSync.setPassageTime(all.get(i).getCrossTime());
                        if (attendanceSync.getFlag().equals("正常")) {
                            attendanceSync.setFlag("1");
                        } else if (attendanceSync.getFlag().equals("异常")) {
                            attendanceSync.setFlag("0");
                        }
                        // 判断考勤是否已经存在，已经存在不会添加
                        QueryWrapper<AttendanceSync> wrapper = new QueryWrapper();
                        wrapper.eq("date_of_attendance",attendanceSync.getDateOfAttendance());
                        wrapper.eq("user_id",attendanceSync.getUserId());
                        List<AttendanceSync> attendanceSyncList = attendanceSyncRepository.selectList(wrapper);
                        if(attendanceSyncList.size()==0)
                            attendanceSyncs.add(attendanceSync);
//                            attendanceSyncRepository.addAttendanceSync(attendanceSync);
                    }
                }
                SysUser user = new SysUser();
                user.setFlag("1");
                user.setAttendanceShow("0");
                    //查询所有在职，显示考勤的用户
                if (StringUtils.isNotBlank(scheduledResult.getPhone())) {
                    user.setPhone(scheduledResult.getPhone());
                }
                List<SysUser> users = sysUserRepositorys.querySysUsers(user);
                List<Integer> userIds = all.stream().map(Attendance::getUserId).collect(Collectors.toList());
                users = users.stream().filter(u -> !userIds.contains(u.getBusinessId())).collect(Collectors.toList());
                for (SysUser sysUser : users) {
                    attendanceSync = new AttendanceSync();
                    attendanceSync.setUserId(sysUser.getBusinessId());
                    attendanceSync.setName(sysUser.getName());
                    attendanceSync.setPhone(sysUser.getPhone());
                    attendanceSync.setDeptId(sysUser.getDeptId());
                    attendanceSync.setDeptLabel(sysUser.getDeptName());
                    attendanceSync.setPosition(sysUser.getPosition());
                    attendanceSync.setPositionLabel(sysUser.getPositionLabel());
                    attendanceSync.setWorkHours(null);
                    attendanceSync.setOffHours(null);
                    attendanceSync.setWorkDeviceName(null);
                    attendanceSync.setOffDeviceName(null);
                    attendanceSync.setDateOfAttendance(sdf.format(scheduledResult.getKqDate()));
                    attendanceSync.setTimeOnJob(null);
                    attendanceSync.setNltBz(sysUser.getNltBz());
                    attendanceSync.setNltBzLabel(sysUser.getNltBzLabel());
                    attendanceSync.setCreateDate(new Date());
                    attendanceSync.setPassageTime(null);
                    attendanceSync.setFlag("0");
                    // 判断考勤是否已经存在，已经存在不会添加
                    QueryWrapper<AttendanceSync> wrapper = new QueryWrapper();
                    wrapper.eq("date_of_attendance",attendanceSync.getDateOfAttendance());
                    wrapper.eq("user_id",attendanceSync.getUserId());
                    List<AttendanceSync> attendanceSyncList = attendanceSyncRepository.selectList(wrapper);
                    if(attendanceSyncList.size()==0)
                        attendanceSyncs.add(attendanceSync);
//                        attendanceSyncRepository.addAttendanceSync(attendanceSync);
                }
            }
        }
        if (!attendanceSyncs.isEmpty())
            attendanceSyncRepository.addAttendanceListSync(attendanceSyncs);
        return new CyPersistModel(1);
    }

    List<Attendance> selectAttendanceList(Attendance attendance, ScheduledResult scheduledResult) throws ParseException {
        List<Attendance> attendanceList = new ArrayList<>();
        // 根据时间范围筛选所有打卡成功的记录
        // 获取前一天的时间
        // TODO: 2023/1/1  临时修改获取当天的数据
//        Date update = TimeUtils.getUpdate();
        attendance.setBeginTime(TimeUtils.getDateNotTime(scheduledResult.getKqDate()));
        attendance.setEndTime(TimeUtils.getDateEndDay(scheduledResult.getKqDate()));
        // 查询时间范围内员工的正常记录集合
        List<DoorRecord> doorRecordList = doorRecordRepository.queryAttendance(attendance);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");//这里可以指定日期的格式
        //  根据用户分组(每个员工时间范围内的所有打卡记录)
//        Map<Integer, List<DoorRecord>> listMap = doorRecordList.stream()
//                .collect(Collectors.groupingBy(DoorRecord::getUserId));
        // todo 2022/12/30 根据用户分组(每个员工时间范围内的所有打卡记录)
        Map<String, List<DoorRecord>> listMap = doorRecordList.stream()
                .collect(Collectors.groupingBy(DoorRecord::getPersonId));
        for (String key : listMap.keySet()) {
            List<DoorRecord> records = listMap.get(key);
            // 某个员工某日所有通行记录
            Attendance att = new Attendance();
            att.setPeopleName(records.get(0).getName());
            att.setPhoneNumber(records.get(0).getMobile());
            att.setDeptName(records.get(0).getDeptName());
            att.setPositionName(records.get(0).getPositionName());
            att.setNltBz(records.get(0).getNltBz());
            att.setKqDate(sdf.format(scheduledResult.getKqDate()));
            att.setUserId(records.get(0).getUserId());
            att = attendanceService.setValue(records, att);
            if (att != null)
                attendanceList.add(att);
        }
        return attendanceList;
    }


    public static List<String> daysBetween(String smdate, String bdate) throws ParseException {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        if (sdf.parse(smdate).after(sdf.parse(bdate)))
            throw new CyServiceException(500,"结束时间不能早于开始时间");
        List<String> list = new ArrayList<>();
        list.add(smdate);
        Calendar calendar = new GregorianCalendar();
        calendar.setTime(sdf.parse(smdate));
        calendar.add(Calendar.DATE,1);
        Date date = calendar.getTime();
        while(!TimeUtils.getDateNotTime(date).after(TimeUtils.getDateNotTime(sdf.parse(bdate)))){
            calendar = new GregorianCalendar();
            calendar.setTime(date);
            calendar.add(Calendar.DATE,1);
            list.add(sdf.format(date));
            date = calendar.getTime();
        }
//        list.add(bdate);
        log.error(""+list);
        return list;
    }

    public static void main(String[] args) throws ParseException {
        daysBetween("2023-01-02","2023-01-01");
    }
}
