package org.rcisoft.sys.doordeviceinfo.controller;


/*固定导入*/

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.core.anno.CyOpeLogAnno;
import org.rcisoft.core.operlog.enums.CyLogTypeEnum;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.springframework.validation.BindingResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.util.CyResultGenUtil;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.constant.CyMessCons;
import org.rcisoft.core.controller.CyPaginationController;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.model.CyGridModel;
import org.rcisoft.core.exception.CyServiceException;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;


import org.rcisoft.sys.doordeviceinfo.entity.DoorDeviceInfo;
import org.rcisoft.sys.doordeviceinfo.service.DoorDeviceInfoService;

import java.util.List;

/**
 * Created by cy on 2022年12月23日 下午2:04:50.
 */
@RestController
@RequestMapping("/doordeviceinfo")
public class DoorDeviceInfoController extends CyPaginationController<DoorDeviceInfo> {


    @Autowired
    private DoorDeviceInfoService doorDeviceInfoServiceImpl;

    //@PreAuthorize("@cyPerm.hasPerm('sys:rDeviceInfo:add')")
    @CyOpeLogAnno(title = "system-门磁设备记录表管理-新增门磁设备记录表", businessType = CyLogTypeEnum.INSERT)
    @ApiOperation(value = "添加门磁设备记录表", notes = "添加门磁设备记录表")
    @PostMapping(value = "/add")
    public CyResult add(@Valid DoorDeviceInfo doorDeviceInfo, BindingResult bindingResult) {
        CyPersistModel data = doorDeviceInfoServiceImpl.persist(doorDeviceInfo);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                doorDeviceInfo);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:rDeviceInfo:delete')")
    @CyOpeLogAnno(title = "system-门磁设备记录表管理-删除门磁设备记录表", businessType = CyLogTypeEnum.DELETE)
    @ApiOperation(value = "删除门磁设备记录表", notes = "删除门磁设备记录表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/delete/{businessId:\\w+}")
    public CyResult delete(@PathVariable int businessId, DoorDeviceInfo doorDeviceInfo) {
        doorDeviceInfo.setBusinessId(businessId);
        CyPersistModel data = doorDeviceInfoServiceImpl.remove(doorDeviceInfo);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                businessId);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:rDeviceInfo:update')")
    @CyOpeLogAnno(title = "system-门磁设备记录表管理-修改门磁设备记录表", businessType = CyLogTypeEnum.UPDATE)
    @ApiOperation(value = "修改门磁设备记录表", notes = "修改门磁设备记录表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PutMapping("/update")
    public CyResult update(@RequestBody DoorDeviceInfo doorDeviceInfo, BindingResult bindingResult) {
//        doorDeviceInfo.setBusinessId(businessId);
        CyPersistModel data = doorDeviceInfoServiceImpl.merge(doorDeviceInfo);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                doorDeviceInfo);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:rDeviceInfo:query')")
    @CyOpeLogAnno(title = "system-门磁设备记录表管理-查询门磁设备记录表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value = "查询单一门磁设备记录表", notes = "查询单一门磁设备记录表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @GetMapping("/detail/{businessId:\\w+}")
    public CyResult detail(@PathVariable int businessId) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                doorDeviceInfoServiceImpl.findById(businessId));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:rDeviceInfo:list')")
    @CyOpeLogAnno(title = "system-门磁设备记录表管理-查询门磁设备记录表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value = "查询门磁设备记录表集合", notes = "查询门磁设备记录表集合")
    @GetMapping(value = "/queryDoorDeviceInfos")
    public CyResult queryDoorDeviceInfos(DoorDeviceInfo doorDeviceInfo) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                doorDeviceInfoServiceImpl.findAll(doorDeviceInfo));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:rDeviceInfo:list')")
    @CyOpeLogAnno(title = "system-门磁设备记录表管理-查询门磁设备记录表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value = "分页查询门磁设备记录表集合", notes = "分页查询门磁设备记录表集合")
    @GetMapping(value = "/queryDoorDeviceInfoByPagination")
    public CyGridModel listByPagination(DoorDeviceInfo doorDeviceInfo) {
        doorDeviceInfoServiceImpl.findAllByPagination(getPaginationUtility(), doorDeviceInfo);
        return getGridModelResponse();
    }

    @CyOpeLogAnno(title = "system-门磁设备记录表管理-查询门磁设备记录表", businessType = CyLogTypeEnum.EXPORT)
    @ApiOperation(value = "导出门磁设备记录表信息", notes = "导出门磁设备记录表信息")
    @GetMapping(value = "/export")
    public CyResult outDoorDeviceInfo(HttpServletResponse response, DoorDeviceInfo doorDeviceInfo, @PathVariable @RequestParam(defaultValue = "0") String excelId) {
        String excelName = "";
        switch (excelId) {
            case "0":
                excelName = "门磁设备记录表信息.xls";
                break;
            case "1":
                excelName = "门磁设备记录表信息.xlsx";
                break;
            case "2":
                excelName = "门磁设备记录表信息.csv";
                break;
        }
        List<DoorDeviceInfo> doorDeviceInfoList = doorDeviceInfoServiceImpl.export(doorDeviceInfo);
        CyEpExcelUtil.exportExcel(doorDeviceInfoList, "门磁设备记录表信息", "门磁设备记录表信息", DoorDeviceInfo.class, excelName, response);
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                doorDeviceInfoList);
    }
}
