package org.rcisoft.sys.doordeviceinfo.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.aop.CyPageUtilAsp;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.rcisoft.sys.doordeviceinfo.dao.DoorDeviceInfoRepository;
import org.rcisoft.sys.doordeviceinfo.entity.DoorDeviceInfo;
import org.rcisoft.sys.doordeviceinfo.service.DoorDeviceInfoService;
import org.rcisoft.core.service.CyBaseService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.rcisoft.core.model.CyPageInfo;

import java.util.List;

import lombok.extern.slf4j.Slf4j;

/**
 * Created by cy on 2022年12月23日 下午2:04:50.
 */
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class DoorDeviceInfoServiceImpl extends ServiceImpl<DoorDeviceInfoRepository, DoorDeviceInfo> implements DoorDeviceInfoService {
    @Autowired
    DoorDeviceInfoRepository doorDeviceInfoRepository;

    /**
     * 保存 门磁设备记录表
     *
     * @param doorDeviceInfo
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(DoorDeviceInfo doorDeviceInfo) {
        //增加操作
        int line = baseMapper.insert(doorDeviceInfo);
        log.debug(CyUserUtil.getAuthenUsername() + "新增了ID为" +
                doorDeviceInfo.getBusinessId() + "的门磁设备记录表信息");
        return new CyPersistModel(line);
    }

    /**
     * 删除 门磁设备记录表
     *
     * @param doorDeviceInfo
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(DoorDeviceInfo doorDeviceInfo) {
        int line = baseMapper.realDelete(doorDeviceInfo);
        log.debug(CyUserUtil.getAuthenUsername() + "删除了ID为" +
                doorDeviceInfo.getBusinessId() + "的门磁设备记录表信息");
        return new CyPersistModel(line);
    }

    /**
     * 修改 门磁设备记录表
     *
     * @param doorDeviceInfo
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(DoorDeviceInfo doorDeviceInfo) {
        int line = doorDeviceInfoRepository.updateDoorDeviceInfos(doorDeviceInfo);
        log.debug(CyUserUtil.getAuthenUsername() + "修改了ID为" + doorDeviceInfo.getBusinessId() + "的门磁设备记录表信息");
        return new CyPersistModel(line);
    }

    /**
     * 根据id查询 门磁设备记录表
     *
     * @param id
     * @return
     */
    @Override
    public DoorDeviceInfo findById(int id) {
        return doorDeviceInfoRepository.queryDoorDeviceInfosById(id);
    }

    /**
     * 分页查询 门磁设备记录表
     *
     * @param doorDeviceInfo
     * @return
     */
    @Override
    public IPage<DoorDeviceInfo> findAllByPagination(CyPageInfo<DoorDeviceInfo> paginationUtility,
                                                     DoorDeviceInfo doorDeviceInfo) {
        return baseMapper.queryDoorDeviceInfosPaged(paginationUtility, doorDeviceInfo);

    }

    /**
     * 查询list 门磁设备记录表
     *
     * @param doorDeviceInfo
     * @return
     */
    @Override
    public List<DoorDeviceInfo> findAll(DoorDeviceInfo doorDeviceInfo) {
        return baseMapper.queryDoorDeviceInfos(doorDeviceInfo);

    }

    /**
     * 导出门磁设备记录表
     *
     * @return
     */
    @Override
    public List<DoorDeviceInfo> export(DoorDeviceInfo doorDeviceInfo) {
        List<DoorDeviceInfo> doorDeviceInfoList = baseMapper.queryDoorDeviceInfos(doorDeviceInfo);
        return doorDeviceInfoList;
    }

    /***
     * 根究设备的IMEI编码获取设备
     * @param deviceImei
     * @return
     */
    @Override
    public DoorDeviceInfo getDeviceInfoByIMEICode(String deviceImei) {
        QueryWrapper<DoorDeviceInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("device_imei", deviceImei);
        queryWrapper.eq("del_flag", "0");
        List<DoorDeviceInfo> doorDeviceInfos = baseMapper.selectList(queryWrapper);
        if (doorDeviceInfos != null && !doorDeviceInfos.isEmpty()) {
            return doorDeviceInfos.get(0);
        } else {
            return null;
        }
    }

}
