package org.rcisoft.sys.doorreportdata.controller;


/*固定导入*/
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.core.anno.CyOpeLogAnno;
import org.rcisoft.core.operlog.enums.CyLogTypeEnum;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.springframework.validation.BindingResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.util.CyResultGenUtil;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.constant.CyMessCons;
import org.rcisoft.core.controller.CyPaginationController;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.model.CyGridModel;
import org.rcisoft.core.exception.CyServiceException;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;


import org.rcisoft.sys.doorreportdata.entity.DoorReportData;
import org.rcisoft.sys.doorreportdata.service.DoorReportDataService;

import java.util.List;

/**
* Created by cy on 2022年12月23日 下午1:57:30.
*/
@RestController
@RequestMapping("/doorreportdata")
public class DoorReportDataController extends CyPaginationController<DoorReportData> {


    @Autowired
    private DoorReportDataService doorReportDataServiceImpl;

    //@PreAuthorize("@cyPerm.hasPerm('sys:rReportData:add')")
    @CyOpeLogAnno(title = "system-门磁设备数据记录表管理-新增门磁设备数据记录表", businessType = CyLogTypeEnum.INSERT)
    @ApiOperation(value="添加门磁设备数据记录表", notes="添加门磁设备数据记录表")
    @PostMapping(value = "/add")
    public CyResult add(@Valid DoorReportData doorReportData, BindingResult bindingResult) {
        CyPersistModel data = doorReportDataServiceImpl.persist(doorReportData);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        doorReportData);
    }
    //@PreAuthorize("@cyPerm.hasPerm('sys:rReportData:delete')")
    @CyOpeLogAnno(title = "system-门磁设备数据记录表管理-删除门磁设备数据记录表", businessType = CyLogTypeEnum.DELETE)
    @ApiOperation(value="删除门磁设备数据记录表", notes="删除门磁设备数据记录表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/delete/{businessId:\\w+}")
    public CyResult delete(@PathVariable int businessId,DoorReportData doorReportData) {
        doorReportData.setBusinessId(businessId);
        CyPersistModel data = doorReportDataServiceImpl.remove(doorReportData);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        businessId);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:rReportData:update')")
    @CyOpeLogAnno(title = "system-门磁设备数据记录表管理-修改门磁设备数据记录表", businessType = CyLogTypeEnum.UPDATE)
    @ApiOperation(value="修改门磁设备数据记录表", notes="修改门磁设备数据记录表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PutMapping("/update/{businessId:\\w+}")
    public CyResult update(@PathVariable int businessId, @Valid DoorReportData doorReportData, BindingResult bindingResult) {
        doorReportData.setBusinessId(businessId);
        CyPersistModel data = doorReportDataServiceImpl.merge(doorReportData);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        doorReportData);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:rReportData:query')")
    @CyOpeLogAnno(title = "system-门磁设备数据记录表管理-查询门磁设备数据记录表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="查询单一门磁设备数据记录表", notes="查询单一门磁设备数据记录表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @GetMapping("/detail/{businessId:\\w+}")
    public CyResult detail(@PathVariable int businessId) {
        long dataId = businessId;
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        doorReportDataServiceImpl.findById(dataId));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:rReportData:list')")
    @CyOpeLogAnno(title = "system-门磁设备数据记录表管理-查询门磁设备数据记录表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="查询门磁设备数据记录表集合", notes="查询门磁设备数据记录表集合")
    @GetMapping(value = "/queryDoorReportDatas")
    public CyResult queryDoorReportDatas(DoorReportData doorReportData) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
            doorReportDataServiceImpl.findAll(doorReportData));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:rReportData:list')")
    @CyOpeLogAnno(title = "system-门磁设备数据记录表管理-查询门磁设备数据记录表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="分页查询门磁设备数据记录表集合", notes="分页查询门磁设备数据记录表集合")
    @GetMapping(value = "/queryDoorReportDataByPagination")
    public CyGridModel listByPagination(DoorReportData doorReportData) {
        doorReportDataServiceImpl.findAllByPagination(getPaginationUtility(), doorReportData);
        return getGridModelResponse();
    }

    @CyOpeLogAnno(title = "system-门磁设备数据记录表管理-查询门磁设备数据记录表", businessType = CyLogTypeEnum.EXPORT)
    @ApiOperation(value = "导出门磁设备数据记录表信息", notes = "导出门磁设备数据记录表信息")
    @GetMapping(value = "/export")
    public CyResult outDoorReportData(HttpServletResponse response,DoorReportData doorReportData,@PathVariable @RequestParam(defaultValue = "0") String excelId) {
        String excelName="";
        switch(excelId){
            case "0": excelName="门磁设备数据记录表信息.xls";break;
            case "1": excelName="门磁设备数据记录表信息.xlsx";break;
            case "2": excelName="门磁设备数据记录表信息.csv";break;
        }
        List<DoorReportData> doorReportDataList = doorReportDataServiceImpl.export(doorReportData);
        CyEpExcelUtil.exportExcel(doorReportDataList, "门磁设备数据记录表信息", "门磁设备数据记录表信息", DoorReportData.class, excelName, response);
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
            doorReportDataList);
    }
}
