package org.rcisoft.sys.doorreportdata.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.aop.CyPageUtilAsp;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.rcisoft.sys.doorreportdata.dao.DoorReportDataRepository;
import org.rcisoft.sys.doorreportdata.entity.DoorReportData;
import org.rcisoft.sys.doorreportdata.service.DoorReportDataService;
import org.rcisoft.core.service.CyBaseService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.rcisoft.core.model.CyPageInfo;

import java.util.List;
import lombok.extern.slf4j.Slf4j;

/**
 * Created by cy on 2022年12月23日 下午1:57:30.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class DoorReportDataServiceImpl extends ServiceImpl<DoorReportDataRepository,DoorReportData>  implements DoorReportDataService {


    /**
    * 保存 门磁设备数据记录表
    * @param doorReportData
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(DoorReportData doorReportData){
        //增加操作
        int line = baseMapper.insert(doorReportData);
        log.debug(CyUserUtil.getAuthenUsername()+"新增了ID为"+
            doorReportData.getBusinessId()+"的门磁设备数据记录表信息");
        return new CyPersistModel(line);
    }

    /**
    * 删除 门磁设备数据记录表
    * @param doorReportData
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(DoorReportData doorReportData){
        int line = baseMapper.realDelete(doorReportData);
        log.debug(CyUserUtil.getAuthenUsername()+"删除了ID为"+
            doorReportData.getBusinessId()+"的门磁设备数据记录表信息");
        return new CyPersistModel(line);
    }
    /**
    * 修改 门磁设备数据记录表
    * @param doorReportData
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(DoorReportData doorReportData){
        int line = baseMapper.updateById(doorReportData);
        log.debug(CyUserUtil.getAuthenUsername()+"修改了ID为"+ doorReportData.getBusinessId()+"的门磁设备数据记录表信息");
        return new CyPersistModel(line);
    }

    /**
    * 根据id查询 门磁设备数据记录表
    * @param id
    * @return
    */
    @Override
    public DoorReportData findById(Long id){
        return baseMapper.selectById(id);
    }

    /**
    * 分页查询 门磁设备数据记录表
    * @param doorReportData
    * @return
    */
    @Override
    public IPage<DoorReportData> findAllByPagination(CyPageInfo<DoorReportData> paginationUtility,
        DoorReportData doorReportData){
        return baseMapper.queryDoorReportDatasPaged(paginationUtility,doorReportData);

    }
    /**
    * 查询list 门磁设备数据记录表
    * @param doorReportData
    * @return
    */
    @Override
    public List<DoorReportData> findAll(DoorReportData doorReportData){
        return baseMapper.queryDoorReportDatas(doorReportData);

    }

    /**
    * 导出门磁设备数据记录表
    * @return
    */
    @Override
    public List<DoorReportData> export(DoorReportData doorReportData) {
        List<DoorReportData> doorReportDataList = baseMapper.queryDoorReportDatas(doorReportData);
        return doorReportDataList;
    }

}
