package org.rcisoft.sys.messageinstance.service.impl;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.sys.messageinstance.dao.MessageInstanceRepository;
import org.rcisoft.sys.messageinstance.entity.MessageInstance;
import org.rcisoft.sys.messageinstance.service.MessageInstanceService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.rcisoft.core.model.CyPageInfo;

import java.util.List;

import lombok.extern.slf4j.Slf4j;

/**
 * Created by cy on 2022年11月16日 下午4:37:10.
 */
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class MessageInstanceServiceImpl extends ServiceImpl<MessageInstanceRepository, MessageInstance> implements MessageInstanceService {


    /**
     * 保存 用户消息表
     *
     * @param messageInstance
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(MessageInstance messageInstance) {
        //增加操作
        messageInstance.setNormal();
        int line = baseMapper.insert(messageInstance);
        log.debug(CyUserUtil.getAuthenUsername() + "新增了ID为" +
                messageInstance.getBusinessId() + "的用户消息表信息");
        return new CyPersistModel(line);
    }

    /**
     * 删除 用户消息表
     *
     * @param messageInstance
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(MessageInstance messageInstance) {
        int line = baseMapper.realDelete(messageInstance);
        log.debug(CyUserUtil.getAuthenUsername() + "删除了ID为" +
                messageInstance.getBusinessId() + "的用户消息表信息");
        return new CyPersistModel(line);
    }

    /**
     * 逻辑删除 用户消息表
     *
     * @param messageInstance
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(MessageInstance messageInstance) {
        messageInstance.setDeleted();
        int line = baseMapper.deleteById(messageInstance);
        log.debug(CyUserUtil.getAuthenUsername() + "逻辑删除了ID为" +
                messageInstance.getBusinessId() + "的用户消息表信息");
        return new CyPersistModel(line);
    }

    /**
     * 修改 用户消息表
     *
     * @param messageInstance
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(MessageInstance messageInstance) {
        int line = baseMapper.updateById(messageInstance);
        log.debug(CyUserUtil.getAuthenUsername() + "修改了ID为" + messageInstance.getBusinessId() + "的用户消息表信息");
        return new CyPersistModel(line);
    }

    /**
     * 根据id查询 用户消息表
     *
     * @param id
     * @return
     */
    @Override
    public MessageInstance findById(int id) {
        MessageInstance messageInstance = baseMapper.selectById(id);
        messageInstance.setReadFlag("1");
        baseMapper.updateById(messageInstance);
        return messageInstance;
    }

    /**
     * 分页查询 用户消息表
     *
     * @param messageInstance
     * @return
     */
    @Override
    public IPage<MessageInstance> findAllByPagination(CyPageInfo<MessageInstance> paginationUtility,
                                                      MessageInstance messageInstance) {
        return baseMapper.queryMessageInstancesPaged(paginationUtility, messageInstance);

    }

    /**
     * 查询list 用户消息表
     *
     * @param messageInstance
     * @return
     */
    @Override
    public List<MessageInstance> findAll(MessageInstance messageInstance) {
        return baseMapper.queryMessageInstances(messageInstance);

    }

}
