package org.rcisoft.sys.sysuser.controller;


/*固定导入*/
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.core.anno.CyOpeLogAnno;
import org.rcisoft.core.operlog.enums.CyLogTypeEnum;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.rcisoft.sys.sysuser.entity.SysDept;
import org.rcisoft.sys.sysuser.service.SysDeptService;
import org.springframework.validation.BindingResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.util.CyResultGenUtil;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.constant.CyMessCons;
import org.rcisoft.core.controller.CyPaginationController;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.model.CyGridModel;
import org.rcisoft.core.exception.CyServiceException;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;



import java.util.List;

/**
* Created by cy on 2022年11月18日 上午9:03:23.
*/
@RestController
@RequestMapping("/sysDept")
public class SysDeptController extends CyPaginationController<SysDept> {


    @Autowired
    private SysDeptService sysDeptServiceImpl;

    //@PreAuthorize("@cyPerm.hasPerm('sys:dept:add')")
    @CyOpeLogAnno(title = "system-部门表管理-新增部门表", businessType = CyLogTypeEnum.INSERT)
    @ApiOperation(value="添加部门表", notes="添加部门表")
    @PostMapping(value = "/add")
    public CyResult add(@RequestBody SysDept sysDept, BindingResult bindingResult) {
        CyPersistModel data = sysDeptServiceImpl.persist(sysDept);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        sysDept);
    }
    //@PreAuthorize("@cyPerm.hasPerm('sys:dept:delete')")
    @CyOpeLogAnno(title = "system-部门表管理-删除部门表", businessType = CyLogTypeEnum.DELETE)
    @ApiOperation(value="逻辑删除部门表", notes="逻辑删除部门表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/deleteLogical/{businessId:\\w+}")
    public CyResult deleteLogical(@PathVariable int businessId,SysDept sysDept) {
        sysDept.setBusinessId(businessId);
        CyPersistModel data = sysDeptServiceImpl.removeLogical(sysDept);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        businessId);
    }
    //@PreAuthorize("@cyPerm.hasPerm('sys:dept:delete')")
    @CyOpeLogAnno(title = "system-部门表管理-删除部门表", businessType = CyLogTypeEnum.DELETE)
    @ApiOperation(value="删除部门表", notes="删除部门表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/delete/{businessId:\\w+}")
    public CyResult delete(@PathVariable int businessId,SysDept sysDept) {
        sysDept.setBusinessId(businessId);
        CyPersistModel data = sysDeptServiceImpl.remove(sysDept);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        businessId);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:dept:update')")
    @CyOpeLogAnno(title = "system-部门表管理-修改部门表", businessType = CyLogTypeEnum.UPDATE)
    @ApiOperation(value="修改部门表", notes="修改部门表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PutMapping("/update")
    public CyResult update(@RequestBody SysDept sysDept, BindingResult bindingResult) {
        CyPersistModel data = sysDeptServiceImpl.merge(sysDept);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        sysDept);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:dept:query')")
    @CyOpeLogAnno(title = "system-部门表管理-查询部门表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="查询单一部门表", notes="查询单一部门表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @GetMapping("/detail/{businessId:\\w+}")
    public CyResult detail(@PathVariable String businessId) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        sysDeptServiceImpl.findById(businessId));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:dept:list')")
    @CyOpeLogAnno(title = "system-部门表管理-查询部门表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="查询部门表集合", notes="查询部门表集合")
    @GetMapping(value = "/querySysDepts")
    public CyResult querySysDepts(SysDept sysDept) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
            sysDeptServiceImpl.findAll(sysDept));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:dept:list')")
    @CyOpeLogAnno(title = "system-部门表管理-查询部门表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="分页查询部门表集合", notes="分页查询部门表集合")
    @GetMapping(value = "/querySysDeptByPagination")
    public CyGridModel listByPagination(SysDept sysDept) {
        sysDeptServiceImpl.findAllByPagination(getPaginationUtility(), sysDept);
        return getGridModelResponse();
    }

    @CyOpeLogAnno(title = "system-部门表管理-查询部门表", businessType = CyLogTypeEnum.EXPORT)
    @ApiOperation(value = "导出部门表信息", notes = "导出部门表信息")
    @GetMapping(value = "/export")
    public CyResult outSysDept(HttpServletResponse response,SysDept sysDept,@PathVariable @RequestParam(defaultValue = "0") String excelId) {
        String excelName="";
        switch(excelId){
            case "0": excelName="部门表信息.xls";break;
            case "1": excelName="部门表信息.xlsx";break;
            case "2": excelName="部门表信息.csv";break;
        }
        List<SysDept> sysDeptList = sysDeptServiceImpl.export(sysDept);
        CyEpExcelUtil.exportExcel(sysDeptList, "部门表信息", "部门表信息", SysDept.class, excelName, response);
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
            sysDeptList);
    }
}
