package org.rcisoft.sys.sysusercard.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.aop.CyPageUtilAsp;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.rcisoft.sys.sysusercard.dao.SysUserCardRepository;
import org.rcisoft.sys.sysusercard.entity.SysUserCard;
import org.rcisoft.sys.sysusercard.service.SysUserCardService;
import org.rcisoft.core.service.CyBaseService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.rcisoft.core.model.CyPageInfo;

import java.util.List;
import lombok.extern.slf4j.Slf4j;

/**
 * Created by cy on Dec 28, 2022, 2:07:15 PM.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class SysUserCardServiceImpl extends ServiceImpl<SysUserCardRepository,SysUserCard>  implements SysUserCardService {


    /**
    * 保存 人员凭证记录
    * @param sysUserCard
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(SysUserCard sysUserCard){
        int line = baseMapper.insert(sysUserCard);
        log.debug(CyUserUtil.getAuthenUsername()+"新增了ID为"+
            sysUserCard.getBusinessId()+"的人员凭证记录表信息");
        return new CyPersistModel(line);
    }

    /**
    * 删除 人员凭证记录
    * @param sysUserCard
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(SysUserCard sysUserCard){
        int line = baseMapper.realDelete(sysUserCard);
        log.debug(CyUserUtil.getAuthenUsername()+"删除了ID为"+
            sysUserCard.getBusinessId()+"的人员凭证记录表信息");
        return new CyPersistModel(line);
    }
    /**
    * 逻辑删除 人员凭证记录
    * @param sysUserCard
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(SysUserCard sysUserCard){
        sysUserCard.setDeleted();
        int line = baseMapper.deleteById(sysUserCard);
            log.debug(CyUserUtil.getAuthenUsername()+"逻辑删除了ID为"+
                sysUserCard.getBusinessId()+"的人员凭证记录表信息");
            return new CyPersistModel(line);
        }
    /**
    * 修改 人员凭证记录
    * @param sysUserCard
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(SysUserCard sysUserCard){
        int line = baseMapper.updateById(sysUserCard);
        log.debug(CyUserUtil.getAuthenUsername()+"修改了ID为"+ sysUserCard.getBusinessId()+"的人员凭证记录表信息");
        return new CyPersistModel(line);
    }

    /**
    * 根据id查询 人员凭证记录
    * @param id
    * @return
    */
    @Override
    public SysUserCard findById(int id){
        return baseMapper.selectById(id);
    }

    /**
    * 分页查询 人员凭证记录
    * @param sysUserCard
    * @return
    */
    @Override
    public IPage<SysUserCard> findAllByPagination(CyPageInfo<SysUserCard> paginationUtility,
        SysUserCard sysUserCard){
        return baseMapper.querySysUserCardsPaged(paginationUtility,sysUserCard);

    }
    /**
    * 查询list 人员凭证记录
    * @param sysUserCard
    * @return
    */
    @Override
    public List<SysUserCard> findAll(SysUserCard sysUserCard){
        return baseMapper.querySysUserCards(sysUserCard);

    }

    /**
    * 导出人员凭证记录表
    * @return
    */
    @Override
    public List<SysUserCard> export(SysUserCard sysUserCard) {
        List<SysUserCard> sysUserCardList = baseMapper.querySysUserCards(sysUserCard);
        return sysUserCardList;
    }

    /**
     * 根据用户ID或捷顺用户ID删除用户凭证记录
     * @param userId
     * @param personId
     * @return
     */
    @Override
    public void delUserCardByUserId(Long userId, String personId) {
        if(userId!=null && StringUtils.isNotBlank(personId)){
            QueryWrapper<SysUserCard> delWrapper = new QueryWrapper<>();
            if(userId!=null ){
                delWrapper.eq("user_id",userId);
            }
            if(StringUtils.isNotBlank(personId)){
                delWrapper.eq("person_id",personId);
            }
            this.baseMapper.delete(delWrapper);
        }
    }

}
