package org.rcisoft.sys.visit.controller;


import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.core.anno.CyOpeLogAnno;
import org.rcisoft.core.constant.CyMessCons;
import org.rcisoft.core.controller.CyPaginationController;
import org.rcisoft.core.model.CyGridModel;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.operlog.enums.CyLogTypeEnum;
import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.util.CyResultGenUtil;
import org.rcisoft.sys.visit.entity.VisitInfoMation;
import org.rcisoft.sys.visit.service.InvitedPersonService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

/**
 * Created by cy laoluo 2022年11月22日
 * 受邀人员请求接口
 */
@RestController
@RequestMapping("/invitedPerson")
@Api(tags = "受邀人员邀请信息")
public class InvitedPersonController extends CyPaginationController<VisitInfoMation> {

    @Autowired
    private InvitedPersonService invitedPersonService;

    /**
     * 根据当前时间获取最近一次的访问安排
     */
    @CyOpeLogAnno(title = "visit-获取最近一次访问安排", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value = "获取最近一次访问安排", notes = "根据当前时间获取最近一次的访问安排")
    @GetMapping("/invitedPerson")
    public CyResult getLastVisitInfo(HttpServletRequest request) {

        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                invitedPersonService.getLastVisitInfo(request));

    }

    /**
     * 根据id获取受邀记录详情信息
     *
     * @param id 受邀记录id
     * @return
     */
    @CyOpeLogAnno(title = "visit-获取受邀记录详情信息", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value = "根据id查找受邀记录详情")
    @GetMapping("/getVisitInfoById/{id:\\w+}")
    public CyResult getVisitInfoById(@PathVariable int id) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                invitedPersonService.getVisitUserById(id));
    }

    /**
     * 根据手机号获取受邀记录详情信息
     * @param visitPhone 访客手机号
     * @return
     */
    @CyOpeLogAnno(title = "visit-获取受邀记录详情信息",businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value = "根据Phone查找受邀记录详情")
    @GetMapping("/getVisitInfoByPhone/{visitPhone:\\w+}")
    public CyResult getVisitInfoById(@PathVariable String visitPhone){
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                invitedPersonService.getVisitUserByPhone(visitPhone));
    }


    /**
     * 根据条件查询邀请记录
     * 分页请求对方法名有要求必须以ByPagination结尾 因为需要对方法名称进行过滤
     */
    @CyOpeLogAnno(title = "visit-根据条件查询邀请记录", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value = "根据条件查询邀请记录")
    @GetMapping("/getVisitInfoListByPagination")
    public CyGridModel getVisitInfoListByPagination(VisitInfoMation visitInfomation ,HttpServletRequest request) {
        invitedPersonService.getVisitInfoListByPagination(getPaginationUtility(), visitInfomation ,request);
        return getGridModelResponse();
    }

}
