package org.rcisoft.sys.visit.controller;


/*固定导入*/
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.core.anno.CyOpeLogAnno;
import org.rcisoft.core.operlog.enums.CyLogTypeEnum;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.rcisoft.sys.visit.entity.VisitUser;
import org.rcisoft.sys.visit.service.VisitUserService;
import org.springframework.validation.BindingResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.util.CyResultGenUtil;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.constant.CyMessCons;
import org.rcisoft.core.controller.CyPaginationController;
import org.rcisoft.core.model.CyGridModel;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.List;

/**
* Created by cy on 2022年11月16日 下午4:30:18.
*/
@RestController
@RequestMapping("/visituser")
public class VisitUserController extends CyPaginationController<VisitUser> {


    @Autowired
    private VisitUserService visitUserServiceImpl;

    //@PreAuthorize("@cyPerm.hasPerm('sys:itUser:add')")
    @CyOpeLogAnno(title = "system-邀访访客信息表管理-新增邀访访客信息表", businessType = CyLogTypeEnum.INSERT)
    @ApiOperation(value="添加邀访访客信息表", notes="添加邀访访客信息表")
    @PostMapping(value = "/add")
    public CyResult add(@Valid VisitUser visitUser, BindingResult bindingResult) {
        CyPersistModel data = visitUserServiceImpl.persist(visitUser);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        visitUser);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:itUser:delete')")
    @CyOpeLogAnno(title = "system-邀访访客信息表管理-删除邀访访客信息表", businessType = CyLogTypeEnum.DELETE)
    @ApiOperation(value="逻辑删除邀访访客信息表", notes="逻辑删除邀访访客信息表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/deleteLogical/{businessId:\\w+}")
    public CyResult deleteLogical(@PathVariable int businessId,VisitUser visitUser) {
        visitUser.setBusinessId(businessId);
        CyPersistModel data = visitUserServiceImpl.removeLogical(visitUser);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        businessId);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:itUser:delete')")
    @CyOpeLogAnno(title = "system-邀访访客信息表管理-删除邀访访客信息表", businessType = CyLogTypeEnum.DELETE)
    @ApiOperation(value="删除邀访访客信息表", notes="删除邀访访客信息表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/delete/{businessId:\\w+}")
    public CyResult delete(@PathVariable int businessId,VisitUser visitUser) {
        visitUser.setBusinessId(businessId);
        CyPersistModel data = visitUserServiceImpl.remove(visitUser);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        businessId);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:itUser:update')")
    @CyOpeLogAnno(title = "system-邀访访客信息表管理-修改邀访访客信息表", businessType = CyLogTypeEnum.UPDATE)
    @ApiOperation(value="修改邀访访客信息表", notes="修改邀访访客信息表")
    @PutMapping("/update")
    public CyResult update(@Valid VisitUser visitUser) {
        CyPersistModel data = visitUserServiceImpl.merge(visitUser);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        visitUser);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:itUser:query')")
    @CyOpeLogAnno(title = "system-邀访访客信息表管理-查询邀访访客信息表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="查询单一邀访访客信息表", notes="查询单一邀访访客信息表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @GetMapping("/detail/{businessId:\\w+}")
    public CyResult detail(@PathVariable int businessId) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        visitUserServiceImpl.findById(businessId));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:itUser:list')")
    @CyOpeLogAnno(title = "system-邀访访客信息表管理-查询邀访访客信息表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="查询邀访访客信息表集合", notes="查询邀访访客信息表集合")
    @GetMapping(value = "/queryVisitUsers")
    public CyResult queryVisitUsers(VisitUser visitUser) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
            visitUserServiceImpl.findAll(visitUser));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:itUser:list')")
    @CyOpeLogAnno(title = "system-邀访访客信息表管理-查询邀访访客信息表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="分页查询邀访访客信息表集合", notes="分页查询邀访访客信息表集合")
    @GetMapping(value = "/queryVisitUserByPagination")
    public CyGridModel listByPagination(VisitUser visitUser) {
        visitUserServiceImpl.findAllByPagination(getPaginationUtility(), visitUser);
        return getGridModelResponse();
    }

    @CyOpeLogAnno(title = "system-邀访访客信息表管理-查询邀访访客信息表", businessType = CyLogTypeEnum.EXPORT)
    @ApiOperation(value = "导出邀访访客信息表信息", notes = "导出邀访访客信息表信息")
    @GetMapping(value = "/export")
    public CyResult outVisitUser(HttpServletResponse response,VisitUser visitUser,@PathVariable @RequestParam(defaultValue = "0") String excelId) {
        String excelName="";
        switch(excelId){
            case "0": excelName="邀访访客信息表信息.xls";break;
            case "1": excelName="邀访访客信息表信息.xlsx";break;
            case "2": excelName="邀访访客信息表信息.csv";break;
        }
        List<VisitUser> visitUserList = visitUserServiceImpl.export(visitUser);
        CyEpExcelUtil.exportExcel(visitUserList, "邀访访客信息表信息", "邀访访客信息表信息", VisitUser.class, excelName, response);
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
            visitUserList);
    }
}
