package org.rcisoft.sys.visit.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.rcisoft.core.component.CyDownErrorTxtComp;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.model.CyPageInfo;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.result.enums.CyResSvcExcEnum;
import org.rcisoft.core.util.CyCompareToolUtils;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.sys.constant.CyDictCons;
import org.rcisoft.sys.dictionary.entity.DictData;
import org.rcisoft.sys.dictionary.service.DictionaryService;
import org.rcisoft.sys.rbac.user.dto.ImportUserDTO;
import org.rcisoft.sys.sysuser.dao.*;
import org.rcisoft.sys.sysuser.dto.ExcelDTO;
import org.rcisoft.sys.sysuser.dto.ExportUserDTO;
import org.rcisoft.sys.sysuser.entity.*;
import org.rcisoft.sys.sysuser.util.CheckUtil;
import org.rcisoft.sys.sysuser.util.CyEpExcelUtil;
import org.rcisoft.sys.visit.dao.VisitInfomationRepository;
import org.rcisoft.sys.visit.dao.VisitPersonRepositorys;
import org.rcisoft.sys.visit.entity.VisitInfoMation;
import org.rcisoft.sys.visit.entity.VisitUser;
import org.rcisoft.sys.visit.service.VisitPersonService;
import org.rcisoft.sys.wbac.user.dto.ErrorDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.Field;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * Created by cy on 2022年11月16日 下午3:19:19.
 */
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class VisitPersonServiceImpl extends ServiceImpl<VisitPersonRepositorys, SysUser> implements VisitPersonService {
    @Autowired
    private VisitPersonRepositorys sysUserRepositorys;
    @Autowired
    private SysDeptRepositorys sysDeptRepositorys;
    @Autowired
    private SysDictDataRepositorys sysDictDataRepositorys;
    @Autowired
    private DictionaryService dictionaryService;
    @Autowired
    private CyDownErrorTxtComp errorTxt;
    @Autowired
    private SysUserRoleRepository sysUserRoleRepository;
    @Autowired
    private VisitInfomationRepository visitInfomationRepository;

    /**
     * 保存 用户表
     *
     * @param sysUser
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(SysUser sysUser) {
        // 根据手机号判断是否存在用户
        List<SysUser> userList = sysUserRepositorys.querySysUsersByPhone(sysUser.getPhone());
        if (userList.size() == 0) {
            // 生成guid
            sysUserRepositorys.addSysUsers(sysUser);
        } else {
            // 判断存在的用户是否员工，员工报错
            for (SysUser u : userList) {
                if (u.getUserType().equals("0"))
                    throw new CyServiceException(500, u.getPhone() + "已被员工绑定,不能成为访客。");
                if (!u.getName().equals(sysUser.getName())) {
                    u.setName(sysUser.getName());
                    sysUserRepositorys.updateSysUser(u);
                }
            }
        }
        return new CyPersistModel(1);
    }

    /**
     * 删除 用户表
     *
     * @param sysUser
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(SysUser sysUser) {
        int line = sysUserRepositorys.delSysUser(Long.valueOf(sysUser.getBusinessId()));
        log.debug(CyUserUtil.getAuthenUsername() + "删除了ID为" +
                sysUser.getBusinessId() + "的用户表信息");
        return new CyPersistModel(line);
    }

    /**
     * 逻辑删除 用户表
     *
     * @param sysUser
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(SysUser sysUser) {
        sysUser.setDeleted();
        int line = baseMapper.deleteById(sysUser);
        log.debug(CyUserUtil.getAuthenUsername() + "逻辑删除了ID为" +
                sysUser.getBusinessId() + "的用户表信息");
        return new CyPersistModel(line);
    }

    /**
     * 修改 用户表
     *
     * @param sysUser
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(SysUser sysUser) {

        //查询是否有重复用户名
        if (sysUserRepositorys.querySysUsersByUserName(sysUser.getUsername()).size() > 1) {
            throw new CyServiceException(500, "用户名已存在");
        }
        //查询已有手机号
//        if (sysUserRepositorys.querySysUsersByPhone(sysUser.getPhone()).size() > 1) {
//            throw new CyServiceException(500, "手机号已存在");
//        }
        int line = sysUserRepositorys.updateSysUser(sysUser);
//        //更新角色
//        sysUserRoleRepository.delRoles(sysUser.getBusinessId());
//        SysUserRole sysUserRole = new SysUserRole();
//        if (sysUser.getRoleList() != null && sysUser.getRoleList().size() > 0) {
//            for (int i = 0; i < sysUser.getRoleList().size(); i++) {
//                sysUserRole.setRoleId((Integer) sysUser.getRoleList().get(i));
//                sysUserRole.setUserId(sysUser.getBusinessId());
//                sysUserRoleRepository.addRoles(sysUserRole);
//            }
//        }

        log.debug(CyUserUtil.getAuthenUsername() + "修改了ID为" + sysUser.getBusinessId() + "的用户表信息");
        return new CyPersistModel(line);
    }

    /**
     * 根据id查询 用户表
     *
     * @param businessId
     * @return
     */
    @Override
    public SysUser findById(Integer businessId) {

        SysUser user = sysUserRepositorys.querySysUsersById(businessId);
        //角色集合转换为数组
        List<SysUserRole> sysUserRoles = sysUserRoleRepository.querySysUserRolesById(businessId);
        if (sysUserRoles.size() != 0) {
            List<Integer> roleIds = sysUserRoles.stream().map(SysUserRole::getRoleId).collect(Collectors.toList());
            user.setRoleList(roleIds);
        }
        //查询上级部门
        if (user.getDeptId() != null) {
            List<SysDept> sysDepts = sysDeptRepositorys.queryDeptBySJ(user.getDeptId());
            Integer[] arr = new Integer[sysDepts.size() + 1];
            if (sysDepts.size() != 0) {
                for (int i = 0; i <= sysDepts.size(); i++) {
                    if (i == sysDepts.size()) {
                        arr[i] = Integer.valueOf(user.getDeptId());
                        break;
                    }
                    arr[i] = sysDepts.get(i).getBusinessId();
                }
                user.setDeptSJ(List.of(arr));
            }
        }


        return user;
    }

    /**
     * 分页查询 用户表
     *
     * @param sysUser
     * @return
     */
    @Override
    public IPage<SysUser> findAllByPagination(CyPageInfo<SysUser> paginationUtility, SysUser sysUser) {
        if (sysUser.getSort() == null || sysUser.getSort().equals("")) {
            sysUser.setSort("1");
        }
        IPage<SysUser> sysUserIPage = baseMapper.querySysUsersPaged(paginationUtility, sysUser);

        List<SysUser> userList = sysUserIPage.getRecords();

        for (SysUser user : userList) {
            String phone = user.getPhone();
            int count = 0;
            VisitInfoMation infoMationQuerry = new VisitInfoMation();
            infoMationQuerry.setVisitStatus("1");
            List<VisitInfoMation> infoMationQuerryList = visitInfomationRepository.queryVisitInfomations(infoMationQuerry);
            for (VisitInfoMation info : infoMationQuerryList) {
                for (VisitUser u : info.getUserList()) {
                    if (u.getVisitPhone() != null && u.getVisitPhone().equals(phone)) {
                        count++;
                    }
                }
            }
            user.setCount(count);
        }

        sysUserIPage.setRecords(userList);

        return sysUserIPage;
    }

    /**
     * 查询list 用户表
     *
     * @param sysUser
     * @return
     */
    @Override
    public List<SysUser> findAll(SysUser sysUser) {
        return baseMapper.querySysUsers(sysUser);

    }

    @Override
    public List<SysUser> getList() {
        List<SysUser> sysUserList = baseMapper.getUserList();
        for (SysUser user : sysUserList) {
            String deptId = user.getDeptId();
            if (deptId != null) {
                SysDept dept = sysDeptRepositorys.getDeptById(deptId);
                if (dept != null && dept.getDeptName() != null) {
                    user.setDeptName(dept.getDeptName());
                }
            }
        }
        return sysUserList;
    }

    public SysUser getUserById(Integer id) {
        SysUser user = baseMapper.getUserById(id);
        String deptId = user.getDeptId();
        if (deptId != null) {
            SysDept dept = sysDeptRepositorys.getDeptById(deptId);
            if (dept != null && dept.getDeptName() != null) {
                user.setDeptName(dept.getDeptName());
            }
        }
        return user;
    }

    public List<SysUser> findAllByData(SysUser sysUser) {
        return sysDictDataRepositorys.queryData(sysUser);

    }

    public List<SysUser> findAllByDept(SysUser sysUser) {
        return sysDeptRepositorys.queryDept(sysUser);

    }

    /**
     * 导出用户表
     *
     * @return
     */
    @Override
    public List export(SysUser sysUser) {
        List<SysUser> sysUserList = baseMapper.querySysUsers(sysUser);
        ArrayList<SysUser> list = new ArrayList<>();
        for (int i = 0; i < sysUserList.size(); i++) {
            String deptId = sysUserList.get(i).getDeptId();
            //根据部门id查询部门表
            List<SysDept> sysDepts = sysDeptRepositorys.queryDeptById(deptId);
            if (sysDepts.size() != 0) {
                //获取部门名称
                String deptName = sysDepts.get(0).getDeptName();
                sysUserList.get(i).setDeptId(deptName);
            }
            //查询职位id
            String position = sysUserList.get(i).getPosition();
            List<SysDictData> sysDictData = sysDictDataRepositorys.queryDataByPosition(position);
            if (sysDictData.size() != 0) {
                //查询职位中文名
                String dictLabel = sysDictData.get(0).getDictLabel();
                sysUserList.get(i).setPosition(dictLabel);
            }
            //查询班次id
            String nltBz = sysUserList.get(i).getNltBz();
            List<SysDictData> sysDictData1 = sysDictDataRepositorys.queryDataByNltBz(nltBz);
            if (sysDictData1.size() != 0) {
                //查询班次中文名
                String dictLabel = sysDictData1.get(0).getDictLabel();
                sysUserList.get(i).setNltBz(dictLabel);
            }
            //性别
            if (sysUserList.get(i).getSex() != null) {
                if (sysUserList.get(i).getSex().equals("1")) {
                    sysUserList.get(i).setSex("男");
                } else if (sysUserList.get(i).getSex().equals("0")) {
                    sysUserList.get(i).setSex("女");
                }
            }
            //身份
            if (sysUserList.get(i).getIdentity() != null) {
                if (sysUserList.get(i).getIdentity().equals("1")) {
                    sysUserList.get(i).setIdentity("正式员工");
                }
            }

            list.add(sysUserList.get(i));
        }
        return list;
    }

    @Override
    public int importSysUsers(List<ExportUserDTO> exportUserDTOS) {

        for (int i = 0; i < exportUserDTOS.size(); i++) {
            ExportUserDTO exportUserDTO1 = exportUserDTOS.get(i);
            sysUserRepositorys.addSysUser(exportUserDTO1);
        }
        return 1;
    }

    @Override
    public int exportEmptyTemplate(HttpServletResponse response) {
        List<ExportUserDTO> imOutSysUsers = new ArrayList<>();
        ExportUserDTO exportUserDTO = new ExportUserDTO();
        for (int i = 0; i < 500; i++) {
            imOutSysUsers.add(exportUserDTO);
        }
        // 导出模板需要的下拉列的值
        ExcelDTO excelDto = new ExcelDTO();
        SysDictData sysDictData = new SysDictData();
        sysDictData.setDictType("shift");
        sysDictData.setNormal();
        List<SysDictData> shiftList = sysDictDataRepositorys.querySysDictDatas(sysDictData);
        String[] shift = shiftList.stream().map(SysDictData::getDictLabel).collect(Collectors.toList())
                .toArray(new String[shiftList.size()]);
        excelDto.setShift(shift);
        CyEpExcelUtil.exportExcelSelect(imOutSysUsers, "用户信息", "用户",
                ExportUserDTO.class, "模板.xls", response, excelDto);
        return 1;
    }

    void errorText(String error, ImportUserDTO importUserDTO, List<Map<Integer, List<String>>> maps, String fileName) {
        List<String> strings = new ArrayList<String>();
        strings.add(error);
        Map<Integer, List<String>> map = new HashMap<>();
        map.put(-1, strings);
        maps.add(map);
        importUserDTO.setFilename(fileName);
    }

    /**
     * @param persionId
     * @param userId
     * @return
     */
    @Override
    public boolean saveJieLinkId(String persionId, Integer userId) {
        /**
         * TODO 此处需要格局用户的主键ID将捷顺的主键ID存储进去
         * 此处未完成
         */
        SysUser modifyData = new SysUser();
        modifyData.setBusinessId(userId);
        modifyData.setPersonId(persionId);
        int i = sysUserRepositorys.saveJieLinkPersionId(modifyData);
        if (i > 0) {
            return true;
        } else {
            return false;
        }
    }

    @Override
    public String getPersionIdByUserId(Integer userId) {
        return sysUserRepositorys.getPersionIdByUserId(userId);
    }

    /**
     * 根据捷顺的ID查询用户信息
     *
     * @param personId
     * @return
     */
    @Override
    public SysUser getUserDataByPersonId(String personId) {
        return sysUserRepositorys.querySysUsersByPersonId(personId);
    }

    /**
     * 去除List中空数据
     *
     * @param list
     * @param <T>
     * @return
     * @throws Exception
     */
    public <T> List<T> ThrowNull(List<T> list) throws Exception {
        List<T> list2 = new ArrayList<>();
        if (list != null) {
            for (T t : list) {
                if (outNull(t) == true) {
                    list2.add(t);
                }
            }
        }
        return list2;
    }

    //只要有一个不为null则添加进去
    public boolean outNull(Object o) throws Exception {
        if (o != null) {
            Class aa = o.getClass();
            Field[] fields = aa.getDeclaredFields();
            for (Field field : fields) {//id
                field.setAccessible(true);
                if (field.get(o) != null && !field.getName().equals("rowNum")) {
                    return true;
                }
            }
        }
        return false;
    }

}
