package org.rcisoft.sys.visit.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.common.component.CyGlobal;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.result.enums.CyResSvcExcEnum;
import org.rcisoft.core.util.CyCompareToolUtils;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.integration.jieLink.dto.JieLinkDoorRecordDto;
import org.rcisoft.sys.sysuser.dao.SysDictDataRepositorys;
import org.rcisoft.sys.sysuser.dao.SysUserRepositorys;
import org.rcisoft.sys.sysuser.entity.SysDictData;
import org.rcisoft.sys.sysuser.entity.SysUser;
import org.rcisoft.sys.sysuser.service.impl.SysUserServiceImpl;
import org.rcisoft.sys.sysuser.util.CheckUtil;
import org.rcisoft.sys.sysuser.util.CyEpExcelUtil;
import org.rcisoft.sys.visit.dao.VisitInfomationRepository;
import org.rcisoft.sys.visit.dao.VisitPersonRepositorys;
import org.rcisoft.sys.visit.dao.VisitUserRepository;
import org.rcisoft.sys.visit.dto.*;
import org.rcisoft.sys.visit.entity.VisitInfoMation;
import org.rcisoft.sys.visit.entity.VisitUser;
import org.rcisoft.sys.visit.service.VisitUserService;
import org.rcisoft.sys.visit.unit.IdCardUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.rcisoft.core.model.CyPageInfo;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import lombok.extern.slf4j.Slf4j;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;

/**
 * Created by cy on 2022年11月16日 下午4:30:18.
 */
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class VisitUserServiceImpl extends ServiceImpl<VisitUserRepository, VisitUser> implements VisitUserService {

    @Autowired
    private PasswordEncoder passwordEncoder;
    @Autowired
    private CyGlobal global;
    @Autowired
    private SysUserRepositorys sysUserRepositorys;
    @Autowired
    private SysDictDataRepositorys dictDataRepositorys;
    @Autowired
    private SysUserServiceImpl sysUserService;

    @Autowired
    private VisitPersonRepositorys visitPersonRepositorys;

    @Autowired
    private VisitInfomationRepository visitInfomationRepository;

    /**
     * 保存 邀访访客信息表
     *
     * @param visitUser
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(VisitUser visitUser) {
        // 根据手机号判断是否存在用户
        SysUser sysUser = new SysUser();
        sysUser.setName(visitUser.getUserName());
        sysUser.setPhone(visitUser.getVisitPhone());
        sysUser.setUsername(visitUser.getVisitPhone());
        sysUser.setNormal();
        List<SysUser> userList = sysUserRepositorys.querySysUsersByPhone(visitUser.getVisitPhone());
        if (userList.size() == 0) {
            sysUserRepositorys.insert(sysUser);
            visitUser.setPersonId(UUID.randomUUID().toString());
            visitUser.setUserId(sysUser.getBusinessId());
        } else {
            // 判断存在的用户是否员工，员工报错
            for (SysUser u : userList) {
                if (u.getUserType().equals("0"))
                    throw new CyServiceException(500, u.getPhone() + "已被员工绑定,不能成为访客。");
                if (u.getBlacklistFlag().toString().equals("1")) {
                    throw new CyServiceException(500, "不能邀请黑名单内的访客!");
                }
                if (!u.getName().equals(sysUser.getName())) {
                    u.setName(sysUser.getName());
                    sysUserRepositorys.updateSysUser(u);
                }
                visitUser.setPersonId(UUID.randomUUID().toString());
                visitUser.setUserId(u.getBusinessId());
                break;
            }
        }
        visitUser.setNormal();
        //增加操作
        int line = baseMapper.insert(visitUser);
        log.debug(CyUserUtil.getAuthenUsername() + "新增了ID为" +
                visitUser.getBusinessId() + "的邀访访客信息表信息");
        return new CyPersistModel(line);
    }

    /**
     * 删除 邀访访客信息表
     *
     * @param visitUser
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(VisitUser visitUser) {
        int line = baseMapper.realDelete(visitUser);
        log.debug(CyUserUtil.getAuthenUsername() + "删除了ID为" +
                visitUser.getBusinessId() + "的邀访访客信息表信息");
        return new CyPersistModel(line);
    }

    /**
     * 逻辑删除 邀访访客信息表
     *
     * @param visitUser
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(VisitUser visitUser) {
        visitUser.setDeleted();
        int line = baseMapper.deleteById(visitUser);
        log.debug(CyUserUtil.getAuthenUsername() + "逻辑删除了ID为" +
                visitUser.getBusinessId() + "的邀访访客信息表信息");
        return new CyPersistModel(line);
    }

    /**
     * 修改 邀访访客信息表
     *
     * @param visitUser
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(VisitUser visitUser) {
        int line = 1;
        // 根据手机号和邀访id查询访客记录
        VisitUser visitUsers = baseMapper.queryByVisitUser(visitUser);
        if (visitUsers == null)
            throw new CyServiceException(500, "您未被邀请");
        if (!visitUsers.getUserName().equals(visitUser.getUserName())){
            visitUser.setBusinessId(visitUsers.getBusinessId());
            line = baseMapper.updateById(visitUser);
        }
        log.debug(CyUserUtil.getAuthenUsername() + "修改了ID为" + visitUser.getBusinessId() + "的邀访访客信息表信息");
        return new CyPersistModel(line);
    }

    /**
     * 根据id查询 邀访访客信息表
     *
     * @param id
     * @return
     */
    @Override
    public VisitUser findById(int id) {
        return baseMapper.selectById(id);
    }

    /**
     * 分页查询 邀访访客信息表
     *
     * @param visitUser
     * @return
     */
    @Override
    public IPage<VisitUser> findAllByPagination(CyPageInfo<VisitUser> paginationUtility,
                                                VisitUser visitUser) {
        return baseMapper.queryVisitUsersPaged(paginationUtility, visitUser);

    }

    /**
     * 查询list 邀访访客信息表
     *
     * @param visitUser
     * @return
     */
    @Override
    public List<VisitUser> findAll(VisitUser visitUser) {
        return baseMapper.queryVisitUsers(visitUser);

    }

    /**
     * 导出邀访访客信息表
     *
     * @return
     */
    @Override
    public List<VisitUser> export(VisitUser visitUser) {
        List<VisitUser> visitUserList = baseMapper.queryVisitUsers(visitUser);
        return visitUserList;
    }

    @Override
    public int deleteByVisitId(int id) {
        return baseMapper.deleteByVisitId(id);
    }

    @Override
    public int exportEmptyTemplate(HttpServletResponse response) {
        List<ExportVisitUserDTO> dtoList = new ArrayList<>();
        ExportVisitUserDTO exportVisitUserDTO = new ExportVisitUserDTO();
        for (int i = 0; i < 500; i++) {
            dtoList.add(exportVisitUserDTO);
        }
        CyEpExcelUtil.exportExcelSelect(dtoList, "访客信息", "访客",
                ExportVisitUserDTO.class, "模板.xls", response, null);
        return 1;
    }

    void errorText(String error, ImportVistUserDTO importUserDTO, List<Map<Integer, List<String>>> maps, String fileName) {
        List<String> strings = new ArrayList<String>();
        strings.add(error);
        Map<Integer, List<String>> map = new HashMap<>();
        map.put(-1, strings);
        maps.add(map);
        importUserDTO.setFilename(fileName);
    }

    @Override
    public ImportVistUserDTO importVisitUserExcel(MultipartFile file, HttpServletResponse response) throws ParseException {
        ImportVistUserDTO importUserDTO = new ImportVistUserDTO();
        ArrayList<VisitPersonDTO> visitPersonList = new ArrayList<>();
        Calendar calendar = Calendar.getInstance();
        String nowTime = new SimpleDateFormat("yyyyMMddHHmmss").format(calendar.getTime());
        String fileName = "用户错误信息" + nowTime + ".txt";
        List<Map<Integer, List<String>>> maps = new ArrayList<>();
        ExportVisitUserDTO exportUserDTO = new ExportVisitUserDTO();
        boolean flag = false;
        try {
            flag = CyEpExcelUtil.isFileInUser(file, exportUserDTO, 2);
        } catch (Exception e) {
            throw new CyServiceException(500, "非模板文件");
        }
        if (!flag) {
            throw new CyServiceException(500, "非模板文件");
        }
        /** 从excel表中读到数据，有一部分为空值 */
        List<ExportVisitUserDTO> sysUserList = CyEpExcelUtil.importExcel(file, 2, 1, ExportVisitUserDTO.class);
        List<ExportVisitUserDTO> addUserList = new ArrayList<>();
        /** 遍历 从Excel表中读到的数据 抛出不合适的数据 */
        //判断整个Excel是不是为空 计算非空数据有多少条
        int information = CyCompareToolUtils.cmpareInformation(sysUserList);
        if (information == 0) {
            throw new CyServiceException(500, "Excel文件不可为空，请检查Excel文件是否填写！");
        } else if (information > 500) {
            throw new CyServiceException(500, "单次导入数据不可超过500，请检查！");
        } else {
            //去除空数据
            try {
                sysUserList = sysUserService.ThrowNull(sysUserList);
            } catch (Exception e) {
                throw new CyServiceException(CyResSvcExcEnum.ERROR_NETWORK);
            }
            for (ExportVisitUserDTO sysUser : sysUserList) {
                //判断 是否为空
                VisitPersonDTO userNull = CheckUtil.visvitNotNull(sysUser);
                List sysUsers = sysUserRepositorys.querySysUsersByPhones(sysUser.getPhone());
                if (userNull != null) {
                    visitPersonList.add(userNull);
                } else {
                    addUserList.add(sysUser);
                    if (sysUsers.size() != 0) {
                        userNull.setError("已有员工绑定此手机号");
                    }
                }

            }
        }
        int i = 0;
        /** 将 信息插入sys_user  */
        if (addUserList.size() > 0) {
            for (ExportVisitUserDTO sysUser : addUserList) {
                i++;

                if (sysUser.getName().length() >= 32) {
                    throw new CyServiceException(1008, "用户名超出32个字符，请重新输入");
                }

                //用户类型
                SysUser user = new SysUser();
                user.setUserType("1");
                Pattern pattern = Pattern.compile("^((13[0-9])|(14[0-9])|(15[0-9])|(16[0-9])|(17[0-9])|(18[0-9]))\\d{8}$+");
                if (sysUser.getPhone().length() != 11) {
                    throw new CyServiceException(1008, "导入的数据第" + i + "行存在手机号格式错误,请重新输入");
                }
                Matcher m = pattern.matcher(sysUser.getPhone());
                boolean isMatch = m.matches();
                if (!isMatch) {
                    throw new CyServiceException(org.rcisoft.sys.visit.entity.CyResSvcExcEnum.PHONE_DATA_ERROR);
                }
                user.setPhone(sysUser.getPhone());
                // 判断当前访客是否存在
                List<SysUser> sysUsers = visitPersonRepositorys.querySysUsers(user);
                user.setPassword(passwordEncoder.encode(global.getResetPassword()));
                user.setName(sysUser.getName());
                user.setUsername(sysUser.getPhone());
                if (sysUsers.size() == 0) {
                    user.setPersonId(UUID.randomUUID().toString());
                    sysUserRepositorys.insert(user);
                } else {
                    sysUsers.get(0).setName(sysUser.getName());
                    sysUserRepositorys.updateSysUser(sysUsers.get(0));
                }
            }
        }
        if (null != visitPersonList && visitPersonList.size() > 0) {
            CyEpExcelUtil.exportExcel(visitPersonList, "访客错误数据", "访客错误数据",
                    VisitPersonDTO.class, "访客错误数据.xlsx", response);
        }
        return null;
    }

    @Async
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    public void updateTest(JieLinkDoorRecordDto recodeDto) {
        // 根据personId 查询用户信息
        SysUser sysUserData = sysUserService.getUserDataByPersonId(recodeDto.getPerson().getPersonId());
        if (sysUserData.getUserType().equals("1")) {
            // 查询当前访问时间阈值
            SysDictData queryDicData = new SysDictData();
            queryDicData.setDictType("timeThreshold");
            queryDicData.setDelFlag("0");
            queryDicData.setFlag("1");
            List<SysDictData> dictDataList = dictDataRepositorys.querySysDictDatas(queryDicData);
            String beginTime = "";
            String endTime = "";
            DateTime crossTime = DateUtil.parse(recodeDto.getCrossTime(), "yyyy-MM-dd HH:mm:ss");
            for (SysDictData dicData : dictDataList) {
                String dictValue = dicData.getDictValue();
                //此处可能会有类型转换报错的BUG
                int factor = Integer.parseInt(dictValue);
                String dictLabel = dicData.getDictLabel();
                if (StringUtils.equals("advance", dictLabel)) {
                    Date startTime = new Date(crossTime.getTime() - (factor * 60 * 1000));
                    beginTime = DateUtil.format(startTime, "yyyy-MM-dd HH:mm:ss");
                } else if (StringUtils.equals("late", dictLabel)) {
                    Date startTime = new Date(crossTime.getTime() + (factor * 60 * 1000));
                    endTime = DateUtil.format(startTime, "yyyy-MM-dd HH:mm:ss");
                }
            }
            // 如果开始时间和结束时间都不为空，查询这段时间内的邀访记录
            // 此处如果数据字典进行变化，可能查不到对应的邀访记录
            VisitUser user = new VisitUser(recodeDto.getPerson().getPersonId(), "0", beginTime, endTime);
            List<VisitInfoMation> visitInfoMationList = visitInfomationRepository.findByUser(user);
            // 更新状态为已到访
            if (visitInfoMationList.size() > 0) {
                visitInfoMationList.forEach(v -> {
                    v.setVisitStatus("1");
                    visitInfomationRepository.updateById(v);
                });
            }
            // 更新最新到访时间
            if (sysUserData.getFirstDate() == null)
                sysUserData.setFirstDate(crossTime);
            sysUserData.setLastDate(crossTime);
            sysUserRepositorys.updateById(sysUserData);
        }
    }
}
